unit UDomino;

interface
uses Math, SysUtils, Graphics;
const MaxN = 28;

type
  TNum = 0..6;
  TSide = (Left,Right,Center);
  TBone = record
    val1:TNum;
    val2:TNum;
    Color:TColor;
    Side:TSide;
  end;

  TSet=array[1..MaxN] of TBone;

  TDomino = class
  private
    FGame:TSet;
    FSize:Integer;
    FLeft:TNum;
    FRight:TNum;
    function GetBone(index: Integer): TBone;
    function CanAdd(var Bone:TBone; Side:TSide):Boolean;
    procedure DelLast;
  public
    constructor Create;
    function Add(Bone:TBone; Side:TSide):Boolean;
    function Add_disordered(Bone:TBone):Boolean;
    procedure Clear;
    function LoadFromFile(FileName:String; ordered:Boolean; nColor:TColor; maxcount: Integer):boolean;
    procedure SaveToFile(FileName:String);
    function NotInGame(Bone:TBone):Boolean;
    function main(Player1,player2:TDomino;Res:TDomino):Boolean;
    property Size:Integer read FSize;
    property Game[index:Integer]:TBone read GetBone;
  end;

  function IsEqualBones(Bone1,Bone2:TBone):Boolean;

implementation

{ TDomino }

function IsEqualBones(Bone1,Bone2:TBone):Boolean;
begin
  result:=(Bone1.val1 = Bone2.val1) and (Bone1.val2 = Bone2.val2) or
          (Bone1.val1 = Bone2.val2) and (Bone1.val2 = Bone2.val1);
end;

function TDomino.NotInGame(Bone: TBone): Boolean;
var
  i:Integer;
begin
  result:=True;
  i:=1;
  while(i<=FSize) and Result do
  begin
    Result:= not IsEqualBones(Bone,FGame[i]);
    inc(i)
  end;

end;

function Empty(Player:TDomino; Game:TDomino):Boolean;
var
  i:Integer;
begin
  Result:= not (Player.Size > Game.Size);
  if Result
    then
      begin
        i:=1;
        while (i<=Player.Size) and Result do
        begin
          Result:= not Game.NotInGame(Player.Game[i]);
          if Result
            then inc(i);
        end;
      end;
end;

procedure SwapBone(var Bone:TBone);
var
  tmp:TNum;
begin
  tmp:=Bone.val1;
  Bone.val1:=Bone.val2;
  Bone.val2:=tmp;
end;

function TDomino.CanAdd(var Bone:TBone; Side:TSide):Boolean;
begin
  Result:=False;
  case Side of
    Left:
      if (Bone.val1 <> Fleft) and (Bone.val2 <> Fleft)
        then Result:=False
        else
          begin
            Result:=True;
            Bone.Side:=Left;
            if (Bone.val1 = FLeFt) and (Bone.val2 <> FLeft)
              then SwapBone(Bone);
          end;
    Right:
      if (Bone.val1 <> FRight) and (Bone.val2 <> FRight)
        then Result:=False
        else
          begin
            Result:=True;
            Bone.Side:=Right;
            if (Bone.val1 <> Fright) and (Bone.val2 = FRight)
              then SwapBone(Bone);
          end;
  end;
end;

function TDomino.Add(Bone: TBone; Side:TSide): Boolean;
begin
  if FSize = 0
    then
      begin
        Result:=True;
        inc(FSize);
        Bone.Side:=Center;
        FGame[FSize]:=Bone;
        FLeft:=Bone.val1;
        FRight:=Bone.val2;
      end
    else
      begin
        Result:= NotInGame(Bone) and CanAdd(Bone,Side);
        if Result
          then
            begin
              inc(FSize);
              FGame[FSize]:=Bone;
              case Side of
                Left: FLeft:=Bone.val1;
                Right: FRight:=Bone.val2;
              end;
            end;
      end;
end;

procedure TDomino.Clear;
begin
  FSize:=0;
end;

constructor TDomino.Create;
begin
  FSize:=0;
end;

procedure TDomino.DelLast;
begin
  if FSize <> 0
    then
      begin
        if (Game[FSize].Side = Left) and (Game[FSize].val1 = FLeft)
          then FLeft:= Game[FSize].val2
          else
            if (Game[FSize].Side = Right) and (Game[FSize].val2 = FRight)
              then FRight:= Game[FSize].val1;
        Dec(FSize);
      end;
end;

function TDomino.main(Player1, player2: TDomino; Res: TDomino): Boolean;
  procedure Backtrack(Player1,player2:TDomino; var dontsteps:Integer);
  var
    i:Integer;
    tmp: Boolean;
    Added:Boolean;
  begin
    if dontsteps = 2
      then
        for i:=1 to FSize do
          Res.Add(Game[i],Game[i].Side)
      else
        begin
          Added:=False;
          i:=1;
          while (i<=Player1.Size) and (dontsteps < 2) do
          begin
            tmp:=Add(Player1.Game[i],Left);
            if tmp
              then
                begin
                  Added:=True;
                  dontsteps:=0;
                  if not Empty(Player1,Self)
                    then Backtrack(player2,Player1,dontsteps);
                  DelLast;
                end;
            tmp:=(dontsteps < 2) and Add(Player1.Game[i],Right);
            if tmp
              then
                begin
                  Added:=True;
                  dontsteps:=0;
                  if not Empty(Player1,Self)
                    then Backtrack(player2,Player1,dontsteps);
                  DelLast;
                end;
            Inc(i);
          end;

          if not Added
            then
              begin
                Inc(dontsteps);
                Backtrack(player2,Player1,dontsteps);
              end
        end;
  end;
var
  DontSteps:Integer;
begin
  DontSteps:=0;
  Backtrack(player1,player2,DontSteps);
  Result:=DontSteps = 2;
end;

function TDomino.GetBone(index: Integer): TBone;
begin
  Result:=FGame[index];
end;

function TDomino.Add_disordered(Bone: TBone): Boolean;
begin
  Result:=NotInGame(Bone);
  if Result
    then
      begin
        inc(FSize);
        FGame[FSize]:=Bone;
        FGame[FSize].Side:=Center;
      end;
end;

function StrToBone(str:string; var Bone:TBone):Boolean;
var
  i:Integer;
  len:Integer;
  tmp:Integer;
begin
  i:=1;
  str:=Trim(str);
  len:=Length(str);
  while (i<=len) and (str[i]<>#9) do inc(i);
  Result := TryStrToInt(Copy(str,1,i-1),tmp) and (tmp>=0) and (tmp<=6);
  if Result
    then
      begin
        Bone.val1:=tmp;
        Delete(str,1,i);
        str:=Trim(str);
        result :=  TryStrToInt(Copy(str,1,Length(str)),tmp) and (tmp>=0) and (tmp<=6);
        if Result
          then Bone.val2:=tmp;
      end;
end;

function TDomino.LoadFromFile(Filename: String; Ordered:Boolean; nColor:TColor; maxcount: Integer): boolean;
var
  F:TextFile;
  str:string;
  bone:TBone;
begin
  AssignFile(F,Filename);
  Reset(F);
  Result:=True;
  while not Eof(F) and (FSize < maxcount) and Result do
  begin
    Readln(F,str);
    Result:=StrToBone(str,bone);
    if Result
      then
        begin
          bone.Color:= nColor;
          if Ordered
            then Result:=Add(bone,Right) or Add(bone,Left)
            else Result:=add_disordered(bone);
        end;
        
  end;
  CloseFile(F);
end;

procedure TDomino.SaveToFile(Filename:String);
var
  F:TextFile;
  i:Integer;
begin
  AssignFile(F,FileName);
  Rewrite(F);
  for i:=1 to FSize do
    Writeln(F,IntToStr(FGame[i].val1)+#9+IntToStr(FGame[i].val2));
  CloseFile(F);
end;

end.
