unit UDrawDomino;

interface
uses UDomino, Graphics, Windows, Grids, ExtCtrls, Forms;
const
  SizeCol = 30;
  r = 3;
  sep = 5;
type
  TDrawDomino = class(TDomino)
  private
   FImg:TImage;
   FCanvas:TCanvas;
   FFileName:string;
   FColor:TColor;
   FModified:Boolean;
  protected
    procedure DrawCell (N: TNum; L,T:Integer; Col:TColor);
  public
    constructor Create(Img:TImage; Col:TColor);
    function Add(Bone:TBone; Side:TSide):Boolean;
    function Add_disordered(Bone:TBone):Boolean;
    function LoadFromFile(FileName:String; ordered:Boolean;nColor:TColor; maxcount: Integer):boolean;
    procedure SaveToFile(FileName:String);
    procedure Draw;
    procedure DrawGame(Player1,Player2, Start:TDrawDomino);
    procedure Clear;
    property Filename : string  read FFileName;
    property Modified : Boolean read FModified write FModified;
    property Color: TColor read FColor;
  end;
implementation

{ TDrawDomino }


constructor TDrawDomino.Create(Img:TImage; Col:TColor);
begin
  inherited Create;
  FFileName:='';
  FModified:=False;
  FImg:=Img;
  FColor:=Col;
  FCanvas:=FImg.Canvas;
  FCanvas.Brush.Color:=clWhite;
  FCanvas.Rectangle(0,0,FImg.Width,FImg.Height);
end;


procedure TDrawDomino.DrawCell (N: TNum; L,T:Integer; Col:TColor);
  procedure makePoint (x,y: integer);
  begin
    FCanvas.Ellipse(x-r, y-r, x+r, y+r)
  end;

begin
  FCanvas.Brush.Color:=clWhite;
  FCanvas.Rectangle(L+1,T+1,L+SizeCol-1,T+SizeCol-1);
  FCanvas.Brush.Color:=Col;
  if n>1 then
    begin
      makePoint(L+Sep,T+Sep);
      makePoint(L+SizeCol-sep,T+SizeCol-Sep);
    end;
  if n>3 then
    begin
      makePoint(L+SizeCol-Sep,T+Sep);
      makePoint(L+Sep,T+SizeCol-Sep);
    end;
  if odd(n) then
    makePoint(L+SizeCol div 2,T+SizeCol div 2);
  if n=6 then
    begin
      makePoint(L+SizeCol div 2,T+Sep);
      makePoint(L+SizeCol div 2,T+SizeCol-Sep)
    end;
end;

procedure TDrawDomino.Draw;
var
  i:Integer;
  LeftCount:Integer;
  LastLeft,LastRight:Integer;
  Position:Integer;
begin
  if Self.Size>0
    then
      begin
        FCanvas.Brush.Color:=clWhite;
        FCanvas.Rectangle(0,0,FImg.Width,FImg.Height);
        LeftCount:=0;
        for i:= 2 to Size do
          if Game[i].Side = UDomino.Left
            then inc (LeftCount);

        LastLeft:=2*LeftCount*SizeCol+5+5*LeftCount;
        LastRight:=2*LeftCount*SizeCol+5+5*LeftCount;
        FCanvas.Pen.Width:=3;
        FCanvas.Brush.Color:=clWhite;
        FCanvas.Pen.Width:=1;
        FCanvas.Brush.Color:=Game[1].Color;
        FCanvas.Rectangle(LastLeft,10,LastLeft+2*SizeCol,10+SizeCol);
        DrawCell(Game[1].val1,LastLeft,10,Game[1].Color);
        DrawCell(Game[1].val2,LastLeft+SizeCol,10,Game[1].Color);
        for i:=2 to Size do
        begin
          if Game[i].Side = Left
            then
              begin
                LastLeft:=LastLeft - 2*SizeCol-5;
                Position:=LastLeft;
              end
            else
              begin
                LastRight:=LastRight + 2*SizeCol+5;
                Position:=LastRight;
              end;
          FCanvas.Pen.Width:=3;
          FCanvas.Brush.Color:=clWhite;
          FCanvas.Pen.Width:=1;
          FCanvas.Brush.Color:=Game[i].Color;
          FCanvas.Rectangle(Position,10,Position+2*SizeCol,10+SizeCol);
          DrawCell(Game[i].val1,Position,10,Game[i].Color);
          DrawCell(Game[i].val2,Position+SizeCol,10,Game[i].Color);
        end;
      end;
end;

procedure TDrawDomino.Clear;
begin
  if Size>0
    then
      begin
        inherited Clear;
        FModified:=True;
        FCanvas.Brush.Color:=clWhite;
        FCanvas.Rectangle(0,0,FImg.Width,FImg.Height);
      end;
end;

function TDrawDomino.Add(Bone: TBone; Side: TSide): Boolean;
begin
  Result := inherited Add(Bone,Side);
  FModified:= FModified or Result;
  Draw;
end;

function TDrawDomino.Add_disordered(Bone: TBone): Boolean;
begin
  Result := inherited Add_disordered(Bone);
  FModified:= FModified or Result;
  Draw;
end;

function TDrawDomino.LoadFromFile(FileName: String; ordered: Boolean;
  nColor:TColor; maxcount: Integer): boolean;
begin
  FFileName:=FileName;
  Result:=inherited LoadFromFile(FileName,ordered,nColor,maxcount);
  FModified:=False;
  Draw;
end;

procedure TDrawDomino.SaveToFile(FileName: String);
begin
  FFileName:=FileName;
  inherited SaveToFile(FileName);
  FModified:=False;
end;

function GetBonePictureCoordinate(Player:TDomino;bone:TBone):Integer;
var
  i:Integer;
  found:Boolean;
begin
  found:=False;
  Result:=5;
  i:=1;
  while (not found) and (i<=Player.Size) do
  begin
    if not IsEqualBones(bone,Player.Game[i])
      then
        begin
          Result:=Result + 2*SizeCol+5;
          Inc(i);
        end
      else found:=True;
  end;
end;

procedure TDrawDomino.DrawGame(Player1, Player2, Start: TDrawDomino);
var
  i:Integer;
  LeftCount:Integer;
  LastLeft,LastRight:Integer;
  PosInPlayerSet:Integer;
  Position:Integer;
begin
  LeftCount:=0;
  for i:= 2 to Size do
    if Game[i].Side = UDomino.Left
      then inc (LeftCount);

  LastLeft:=2*LeftCount*SizeCol+5+5*LeftCount;
  LastRight:=2*LeftCount*SizeCol+5+5*LeftCount;
  FCanvas.Pen.Width:=3;
  FCanvas.Brush.Color:=clWhite;
  FCanvas.Pen.Width:=1;
  FCanvas.Brush.Color:=Game[1].Color;
  FCanvas.Rectangle(LastLeft,10,LastLeft+2*SizeCol,10+SizeCol);
  DrawCell(Game[1].val1,LastLeft,10,Game[1].Color);
  DrawCell(Game[1].val2,LastLeft+SizeCol,10,Game[1].Color);
  for i:=2 to Start.Size do
  begin
    if Game[i].Side = Left
      then
        begin
          LastLeft:=LastLeft - 2*SizeCol-5;
          Position:=LastLeft;
        end
      else
        begin
          LastRight:=LastRight + 2*SizeCol+5;
          Position:=LastRight;
        end;
    FCanvas.Pen.Width:=3;
    FCanvas.Brush.Color:=clWhite;
    FCanvas.Pen.Width:=1;
    FCanvas.Brush.Color:=Game[i].Color;
    FCanvas.Rectangle(Position,10,Position+2*SizeCol,10+SizeCol);
    DrawCell(Game[i].val1,Position,10,Game[i].Color);
    DrawCell(Game[i].val2,Position+SizeCol,10,Game[i].Color);
  end;
  if (Start.Size < Size)
    then
      for i:=Start.Size+1 to Size do
      begin
        if Game[i].Color = Player1.Color
          then
            begin
              PosInPlayerSet:=GetBonePictureCoordinate(Player1,Game[i]);
              Player1.FCanvas.Brush.Color:=clWhite;
              Player1.FCanvas.Rectangle(PosInPlayerSet,10,PosInPlayerSet+2*SizeCol,10+SizeCol);
            end
          else
            begin
              PosInPlayerSet:=GetBonePictureCoordinate(Player2,Game[i]);
              Player2.FCanvas.Brush.Color:=clWhite;
              Player2.FCanvas.Rectangle(PosInPlayerSet,10,PosInPlayerSet+2*SizeCol,10+SizeCol);
            end;
        Application.ProcessMessages;
        Sleep(300);
        if Game[i].Side = Left
          then
            begin
              LastLeft:=LastLeft - 2*SizeCol-5;
              Position:=LastLeft;
            end
          else
            begin
              LastRight:=LastRight + 2*SizeCol+5;
              Position:=LastRight;
            end;
        FCanvas.Pen.Width:=3;
        FCanvas.Brush.Color:=clWhite;
        FCanvas.Pen.Width:=1;
        FCanvas.Brush.Color:=Game[i].Color;
        FCanvas.Rectangle(Position,10,Position+2*SizeCol,10+SizeCol);
        DrawCell(Game[i].val1,Position,10,Game[i].Color);
        DrawCell(Game[i].val2,Position+SizeCol,10,Game[i].Color);
        Application.ProcessMessages;
        Sleep(300);
      end;
end;

end.
